@@ -42,6 +42,7 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* what db package are we using? */
 #undef SASL_GDBM
 #undef SASL_NDBM
+#undef SASL_BERKELEYDB
 
 /* This is where plugins will live at runtime */
 #undef PLUGINDIR
@@ -50,7 +51,8 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #undef WITH_SYMBOL_UNDERSCORE
 
 /* Do we have rc4 available? */
-#undef WITH_RC4
+#undef WITH_CMU_RC4
+#undef WITH_SSL_RC4
 
 /* do we have des available? */
 #undef WITH_DES
@@ -61,6 +63,12 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* do we have PAM for plaintext password checking? */
 #undef HAVE_PAM
 
+/* what flavor of GSSAPI are we using? */
+#undef HAVE_GSS_C_NT_HOSTBASED_SERVICE
+
+/* do we have gssapi.h or gssapi/gssapi.h? */
+#undef HAVE_GSSAPI_H
+
 /* do we have getsubopt()? */
 #undef HAVE_GETSUBOPT
 
