@@ -47,12 +47,12 @@ typedef int sasl_getcallback_t(sasl_conn_t *conn,
 			       void **pcontext);
 
 #ifdef WIN32
-//need to handle the fact that errno has been defined as a function
-//in a dll, not an extern int
+/* need to handle the fact that errno has been defined as a function
+   in a dll, not an extern int */
 #ifdef errno
 #undef errno
-#endif //errno
-#endif //WIN32
+#endif /* errno */
+#endif /* WIN32 */
 
 /* utility function set for plug-ins
  */
