@@ -377,6 +377,11 @@ static int sasl_pam_conv(int num_msg, const struct pam_message **msg,
     int i;
     int ret;
 
+    if (pd == NULL) {
+	/* solaris bug? */
+	return PAM_CONV_ERR;
+    }
+
     reply = (struct pam_response *) sasl_ALLOC(sizeof(struct pam_response) * 
 					       num_msg);
     if (reply == NULL)
@@ -750,7 +755,7 @@ int _sasl_pwcheck_verify_password(sasl_conn_t *conn,
 
     if (reply) { *reply = NULL; }
 
-    strcpy(pwpath, "PWCHECKDIR");
+    strcpy(pwpath, PWCHECKDIR);
     strcat(pwpath, "/pwcheck");
 
     s = socket(AF_UNIX, SOCK_STREAM, 0);
