@@ -43,8 +43,6 @@ SOFTWARE.
 #endif /* errno */
 #endif /* WIN32 */
 
-int _sasl_debug = 0;
-
 static const char build_ident[] = "$Build: libsasl " PACKAGE "-" VERSION " $";
 
 void (*_sasl_client_cleanup_hook)(void) = NULL;
@@ -198,10 +196,11 @@ int _sasl_conn_init(sasl_conn_t *conn,
   conn->got_ip_local = 0;
   conn->got_ip_remote = 0;
   conn->props.min_ssf = 0;
-  if (secflags & SASL_SECURITY_LAYER)
-    conn->props.max_ssf = UINT_MAX;
-  else
-    conn->props.max_ssf = 0;
+  if (secflags & SASL_SECURITY_LAYER) {
+      conn->props.max_ssf = INT_MAX;
+  } else {
+      conn->props.max_ssf = 0;
+  }
   conn->props.security_flags = 0;
   conn->idle_hook = idle_hook;
   conn->callbacks = callbacks;
@@ -443,9 +442,6 @@ _sasl_global_getopt(void *context,
   const sasl_global_callbacks_t * global_callbacks;
   const sasl_callback_t *callback;
 
-  if (! context)
-    return SASL_FAIL;
-
   global_callbacks = (const sasl_global_callbacks_t *) context;
 
   if (global_callbacks && global_callbacks->callbacks)
@@ -655,8 +651,13 @@ _sasl_getcallback(sasl_conn_t * conn,
     /* Nothing ever gets to provide this */
     return SASL_FAIL;
   case SASL_CB_GETOPT:
-    *pproc = &_sasl_conn_getopt;
-    *pcontext = conn;
+      if (conn) {
+	  *pproc = &_sasl_conn_getopt;
+	  *pcontext = conn;
+      } else {
+	  *pproc = &_sasl_global_getopt;
+	  *pcontext = NULL;
+      }
     return SASL_OK;
   }
 
