@@ -76,6 +76,10 @@ getsecret(void *context __attribute__((unused)),
   size_t key_len;
   DBM *db;
   datum dkey, dvalue;
+  void *cntxt;
+  sasl_getopt_t *getopt;
+  const char *path = SASL_DB_PATH;
+  sasl_conn_t *conn = context;
 
   if (! mechanism || ! auth_identity || ! secret || ! realm || ! db_ok)
     return SASL_FAIL;
@@ -85,7 +89,15 @@ getsecret(void *context __attribute__((unused)),
   if (result != SASL_OK)
     return result;
 
-  db = dbm_open(SASL_DB_PATH, O_RDONLY, S_IRUSR | S_IWUSR);
+  if (_sasl_getcallback(conn, SASL_CB_GETOPT,
+                        &getopt, &cntxt) == SASL_OK) {
+      const char *p;
+      if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	  && p != NULL && *p != 0) {
+          path = p;
+      }
+  }
+  db = dbm_open(path, O_RDONLY, S_IRUSR | S_IWUSR);
   if (! db) {
     return SASL_FAIL;
   }
@@ -123,7 +135,7 @@ getsecret(void *context __attribute__((unused)),
 }
 
 static int
-putsecret(void *context __attribute__((unused)),
+putsecret(void *context,
 	  const char *mechanism,
 	  const char *auth_identity,
 	  const char *realm,
@@ -134,6 +146,10 @@ putsecret(void *context __attribute__((unused)),
   size_t key_len;
   DBM *db;
   datum dkey;
+  void *cntxt;
+  sasl_getopt_t *getopt;
+  const char *path = SASL_DB_PATH;
+  sasl_conn_t *conn = context;
 
   if (! mechanism || ! auth_identity || ! realm)
     return SASL_FAIL;
@@ -143,7 +159,16 @@ putsecret(void *context __attribute__((unused)),
   if (result != SASL_OK)
     return result;
 
-  db = dbm_open(SASL_DB_PATH,
+  if (_sasl_getcallback(conn, SASL_CB_GETOPT,
+                        &getopt, &cntxt) == SASL_OK) {
+      const char *p;
+      if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	  && p != NULL && *p != 0) {
+          path = p;
+      }
+  }
+
+  db = dbm_open(path,
 		O_RDWR | O_CREAT /* TODO: what should this be? */,
 		S_IRUSR | S_IWUSR);
   if (! db) {
@@ -180,26 +205,40 @@ sasl_server_putsecret_t *_sasl_db_putsecret = &putsecret;
 
 int _sasl_server_check_db(const sasl_callback_t *verifyfile_cb)
 {
+    const char *path = SASL_DB_PATH;
+    void *cntxt;
+    sasl_getopt_t *getopt;
     int ret = SASL_OK;
-    char *db = sasl_ALLOC(strlen(SASL_DB_PATH) + SUFLEN);
+    char *db;
+
+    if (_sasl_getcallback(NULL, SASL_CB_GETOPT,
+			  &getopt, &cntxt) == SASL_OK) {
+	const char *p;
+	if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	    && p != NULL && *p != 0) {
+	    path = p;
+	}
+    }
+
+    db = sasl_ALLOC(strlen(path) + SUFLEN);
 
     if (db == NULL) {
 	ret = SASL_NOMEM;
     }
 #ifdef DBM_SUFFIX
     if (ret == SASL_OK) {
-	sprintf(db, "%s%s", SASL_DB_PATH, DBM_SUFFIX);
+	sprintf(db, "%s%s", path, DBM_SUFFIX);
 	ret = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(
 	    verifyfile_cb->context, db);
     }
 #else
     if (ret == SASL_OK) {
-	sprintf(db, "%s.dir", SASL_DB_PATH);
+	sprintf(db, "%s.dir", path);
 	ret = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(
 	    verifyfile_cb->context, db);
     }
     if (ret == SASL_OK) {
-	sprintf(db, "%s.pag", SASL_DB_PATH);
+	sprintf(db, "%s.pag", path);
 	ret = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(
 	    verifyfile_cb->context, db);
     }
