@@ -1,6 +1,6 @@
 /* SASL server API implementation
  * Tim Martin
- * $Id: server.c,v 1.61 1999/12/02 19:12:06 leg Exp $
+ * $Id: server.c,v 1.64 1999/12/31 04:39:50 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -399,14 +399,14 @@ static void server_dispose(sasl_conn_t *pconn)
 
 static int init_mechlist(void)
 {
-  /* set util functions - need to do rest*/
-  mechlist->utils=_sasl_alloc_utils(NULL, &global_callbacks);
-  mechlist->utils->checkpass = &_sasl_checkpass;
-  
-  if (mechlist->utils==NULL)
-    return SASL_NOMEM;
+    /* set util functions - need to do rest*/
+    mechlist->utils = _sasl_alloc_utils(NULL, &global_callbacks);
+    if (mechlist->utils == NULL)
+	return SASL_NOMEM;
 
-  return SASL_OK;
+    mechlist->utils->checkpass = &_sasl_checkpass;
+
+    return SASL_OK;
 }
 
 /*
@@ -763,9 +763,17 @@ static int mech_permitted(sasl_conn_t *conn,
 	    || ! conn->external.auth_id)
 	    return 0;
     } else {
+	sasl_ssf_t minssf;
+
+	if (conn->props.min_ssf < conn->external.ssf) {
+	    minssf = 0;
+	} else {
+	    minssf = conn->props.min_ssf - conn->external.ssf;
+	}
+
 	/* Generic mechanism */
-	if (plug->max_ssf < conn->props.min_ssf)
-	    return 0;
+	if (plug->max_ssf < minssf)
+	    return 0; /* too weak */
     }
 
     /* if there are no users in the secrets database we can't use this 
@@ -780,7 +788,7 @@ static int mech_permitted(sasl_conn_t *conn,
 
     /* if there's an external layer this is no longer plaintext */
     if ((conn->props.min_ssf <= conn->external.ssf) && 
-	(conn->external.ssf > 0)) {
+	(conn->external.ssf > 1)) {
 	myflags &= ~SASL_SEC_NOPLAINTEXT;
     }
 
