@@ -86,11 +86,6 @@ extern int      strcasecmp(const char *s1, const char *s2);
 static const char rcsid[] = "$Implementation: Carnegie Mellon SASL "
 VERSION " $";
 
-#ifdef L_DEFAULT_GUARD
-#undef L_DEFAULT_GUARD
-#define L_DEFAULT_GUARD (0)
-#endif
-
 /* external definitions */
 
 #ifdef sun
@@ -113,7 +108,6 @@ typedef unsigned char HASH[HASHLEN + 1];
 #define HASHHEXLEN 32
 typedef unsigned char HASHHEX[HASHHEXLEN + 1];
 
-
 #define CIPHER_DES   2
 #define CIPHER_3DES  4
 #define CIPHER_RC4   8
@@ -159,6 +153,8 @@ typedef struct context {
   int state;			/* state in the authentication we are in */
   int i_am;			/* are we the client or server? */
 
+  sasl_ssf_t limitssf, requiressf; /* application defined bounds, for the
+				      server */
   unsigned char  *nonce;
   int             noncelen;
 
@@ -222,6 +218,17 @@ typedef struct context {
 
 } context_t;
 
+struct digest_cipher {
+    char *name;
+    sasl_ssf_t ssf;
+    int n; /* bits to make privacy key */
+    int flag; /* the flag name of this cipher */
+    
+    cipher_function_t *cipher_enc;
+    cipher_function_t *cipher_dec;
+    cipher_init_t *cipher_init;
+};
+
 /* this is from the rpc world */
 #define IN
 #define OUT
@@ -960,13 +967,14 @@ digest_strdup(sasl_utils_t * utils, const char *in, char **out, int *outlen)
  *****************************/
 
 
-static int dec_3des(context_t *text,
+static int dec_3des(void *v,
 		   const char *input,
 		   unsigned inputlen,
 		   unsigned char digest[16],
 		   char *output,
 		   unsigned *outputlen)
 {
+    context_t *text = (context_t *) v;
     unsigned int lup;
 
     for (lup=0;lup<inputlen;lup+=8)
@@ -1000,13 +1008,14 @@ static int dec_3des(context_t *text,
     return SASL_OK;
 }
 
-int enc_3des(struct context *text,
+int enc_3des(void *v,
 	     const char *input,
 	     unsigned inputlen,
 	     unsigned char digest[16],
 	     char *output,
 	     unsigned *outputlen)
 {
+  context_t *text = (context_t *) v;
   int len;
   int lup;
   int paddinglen;
@@ -1046,7 +1055,7 @@ int enc_3des(struct context *text,
   return SASL_OK;
 }
 
-static int init_3des(context_t *text, 
+static int init_3des(void *v, 
 		     sasl_utils_t *utils __attribute__((unused)), 
 		     char enckey[16],
 		     char deckey[16])
@@ -1054,6 +1063,8 @@ static int init_3des(context_t *text,
 
 
 {
+    context_t *text = (context_t *) v;
+
     des_key_sched((des_cblock *) enckey, text->keysched_enc);
     des_key_sched((des_cblock *) deckey, text->keysched_dec);
     
@@ -1070,13 +1081,14 @@ static int init_3des(context_t *text,
  *
  *****************************/
 
-static int dec_des(context_t *text,
+static int dec_des(void *v, 
 		   const char *input,
 		   unsigned inputlen,
 		   unsigned char digest[16],
 		   char *output,
 		   unsigned *outputlen)
 {
+  context_t *text = (context_t *) v;
   unsigned int lup;
 
   for (lup=0;lup<inputlen;lup+=8)
@@ -1097,13 +1109,14 @@ static int dec_des(context_t *text,
   return SASL_OK;
 }
 
-static int enc_des(context_t *text,
+static int enc_des(void *v, 
 		   const char *input,
 		   unsigned inputlen,
 		   unsigned char digest[16],
 		   char *output,
 		   unsigned *outputlen)
 {
+  context_t *text = (context_t *) v;
   int len;
   int lup;
   int paddinglen;
@@ -1132,11 +1145,13 @@ static int enc_des(context_t *text,
   return SASL_OK;
 }
 
-static int init_des(context_t *text, 
+static int init_des(void *v,
 		    sasl_utils_t *utils __attribute__((unused)), 
 		    char enckey[16],
 		    char deckey[16])
 {
+    context_t *text = (context_t *) v;
+
     des_key_sched((des_cblock *) enckey, text->keysched_enc);
     des_key_sched((des_cblock *) deckey, text->keysched_dec);
     
@@ -1169,13 +1184,15 @@ init_rc4(void *v,
 }
 
 static int
-dec_rc4(context_t *text,
+dec_rc4(void *v,
 	const char *input,
 	unsigned inputlen,
 	unsigned char digest[16],
 	char *output,
 	unsigned *outputlen)
 {
+    context_t *text = (context_t *) v;
+
   /* decrypt the text part */
   rc4_decrypt(text->rc4_dec_context, input, output, inputlen-10);
 
@@ -1189,13 +1206,15 @@ dec_rc4(context_t *text,
 }
 
 static int
-enc_rc4(context_t *text,
+enc_rc4(void *v,
 	const char *input,
 	unsigned inputlen,
 	unsigned char digest[16],
 	char *output,
 	unsigned *outputlen)
 {
+  context_t *text = (context_t *) v;
+
   /* pad is zero */
   *outputlen = inputlen+10;
 
@@ -1235,48 +1254,66 @@ init_rc4(void *v,
 }
 
 static int
-dec_rc4(context_t *text,
+dec_rc4(void *v, 
 	const char *input,
 	unsigned inputlen,
 	unsigned char digest[16],
 	char *output,
 	unsigned *outputlen)
 {
-  /* decrypt the text part */
-  RC4(text->rc4_dec_context, inputlen-10, (unsigned char *) input, output);
+    context_t *text = (context_t *) v;
 
-  /* decrypt the HMAC part */
-  RC4(text->rc4_dec_context, 10, (unsigned char *) input+(inputlen-10), (char *) digest);
+    /* decrypt the text part */
+    RC4(text->rc4_dec_context, inputlen-10, (unsigned char *) input, output);
 
-  /* no padding so we just subtract the HMAC to get the text length */
-  *outputlen=inputlen-10;
+    /* decrypt the HMAC part */
+    RC4(text->rc4_dec_context, 10, (unsigned char *) input+(inputlen-10), 
+	(char *) digest);
 
-  return SASL_OK;
+    /* no padding so we just subtract the HMAC to get the text length */
+    *outputlen=inputlen-10;
+
+    return SASL_OK;
 }
 
 static int
-enc_rc4(context_t *text,
+enc_rc4(void *v,
 	const char *input,
 	unsigned inputlen,
 	unsigned char digest[16],
 	char *output,
 	unsigned *outputlen)
 {
-  /* pad is zero */
-  *outputlen = inputlen+10;
+    context_t *text = (context_t *) v;
 
-  /* encrypt the text part */
-  RC4(text->rc4_enc_context, inputlen, (unsigned char *) input, output);
+    /* pad is zero */
+    *outputlen = inputlen+10;
 
-  /* encrypt the HMAC part */
-  RC4(text->rc4_enc_context, 10, (unsigned char *) digest, (output)+inputlen);
+    /* encrypt the text part */
+    RC4(text->rc4_enc_context, inputlen, (unsigned char *) input, output);
 
-  return SASL_OK;
+    /* encrypt the HMAC part */
+    RC4(text->rc4_enc_context, 10, (unsigned char *) digest, 
+	(output)+inputlen);
+
+    return SASL_OK;
 }
 
 #endif /* WITH_SSL_RC4 */
 
-
+struct digest_cipher available_ciphers[] =
+{
+#ifdef WITH_RC4
+    { "rc4-40", 40, 5, CIPHER_RC440, &enc_rc4, &dec_rc4, &init_rc4 },
+    { "rc4-56", 56, 7, CIPHER_RC456, &enc_rc4, &dec_rc4, &init_rc4 },
+    { "rc4", 128, 16, CIPHER_RC4, &enc_rc4, &dec_rc4, &init_rc4 },
+#endif
+#ifdef WITH_DES
+    { "des", 55, 16, CIPHER_DES, &enc_des, &dec_des, &init_des },
+    { "3des", 112, 16, CIPHER_3DES, &enc_3des, &dec_3des, &init_3des },
+#endif
+    { NULL, 0, 0, 0, NULL, NULL, NULL }
+};
 
 static int create_layer_keys(context_t *text,sasl_utils_t *utils,HASH key, int keylen,
 			     char enckey[16], char deckey[16])
@@ -1914,28 +1951,55 @@ server_continue_step(void *conn_context,
     char           *challenge = NULL;
     char           *realm;
     unsigned char  *nonce;
-
-
-#ifdef WITH_RC4
-#ifdef WITH_DES
-    char           *qop = "auth,auth-int,auth-conf";
-    char           *cipheropts="3des,des,rc4,rc4-40,rc4-56";
-#else
-    char           *qop = "auth,auth-int,auth-conf";
-    char           *cipheropts="rc4,rc4-40,rc4-56";
-#endif /* WITH_DES */
-#else
-#ifdef WITH_DES
-    char           *qop = "auth,auth-int,auth-conf";
-    char           *cipheropts="3des,des";
-#else
-    char           *qop = "auth,auth-int";
-    char           *cipheropts="";
-#endif /* WITH_DES */
-#endif /* WITH_RC4 */
-
     char           *charset = "utf-8";
 
+    char qop[1024], cipheropts[1024];
+    struct digest_cipher *cipher;
+    int added_conf = 0;
+
+    if (sparams->props.max_ssf < sparams->external_ssf) {
+	text->limitssf = 0;
+    } else {
+	text->limitssf = sparams->props.max_ssf - sparams->external_ssf;
+    }
+    if (sparams->props.min_ssf < sparams->external_ssf) {
+	text->requiressf = 0;
+    } else {
+	text->requiressf = sparams->props.min_ssf - sparams->external_ssf;
+    }
+
+    /* what options should we offer the client? */
+    qop[0] = '\0';
+    cipheropts[0] = '\0';
+    if (text->requiressf == 0) {
+	if (*qop) strcat(qop, ",");
+	strcat(qop, "auth");
+    }
+    if (text->requiressf <= 1 && text->limitssf >= 1) {
+	if (*qop) strcat(qop, ",");
+	strcat(qop, "auth-int");
+    }
+    
+    cipher = available_ciphers;
+    while (cipher->name) {
+	/* do we allow this particular cipher? */
+	if (text->requiressf <= cipher->ssf && text->limitssf >= cipher->ssf) {
+	    if (!added_conf) {
+		if (*qop) strcat(qop, ",");
+		strcat(qop, "auth-conf");
+		added_conf = 1;
+	    }
+	    if (*cipheropts) strcat(cipheropts, ",");
+	    strcat(cipheropts, cipher->name);
+	}
+	cipher++;
+    }
+
+    if (*qop == '\0') {
+	/* we didn't allow anything?!? we'll return SASL_TOOWEAK, since
+	 that's close enough */
+	return SASL_TOOWEAK;
+    }
 
     /*
      * digest-challenge  = 1#( realm | nonce | qop-options | stale | maxbuf |
@@ -2198,77 +2262,64 @@ server_continue_step(void *conn_context,
     }
 
     /* defaulting qop to "auth" if not specified */
-    if (qop == NULL)
-      digest_strdup(sparams->utils, "auth", &qop, NULL);      
+    if (qop == NULL) {
+	digest_strdup(sparams->utils, "auth", &qop, NULL);      
+    }
 
     /* check which layer/cipher to use */
-    if (strcmp(qop, "auth-conf") == 0) {
+    if ((!strcmp(qop, "auth-conf")) && (cipher != NULL)) {
+	/* see what cipher was requested */
+	struct digest_cipher *cptr;
 
-#ifdef WITH_DES
-      /* for when privacy supported */
-      VL(("Client requested privacy layer\n"));
-      VL(("Client cipher=%s\n",cipher));
-      if (strcmp(cipher,"des")==0)
-      {
-	text->cipher_enc=(cipher_function_t *) &enc_des;
-	text->cipher_dec=(cipher_function_t *) &dec_des;
-	text->cipher_init=(cipher_init_t *) &init_des;	
-	oparams->mech_ssf = 55;
-	n=16; /* number of bits to make privacy key */
+	VL(("Client requested privacy layer\n"));
+	VL(("Client cipher=%s\n",cipher));
 
-      } else if (strcmp(cipher,"3des")==0) {
-	text->cipher_enc=(cipher_function_t *) &enc_3des;
-	text->cipher_dec=(cipher_function_t *) &dec_3des;
-	text->cipher_init=(cipher_init_t *) &init_3des;
-	oparams->mech_ssf = 112;
-	n=16;
-#else
-      if (0) {
-#endif /* WITH_DES */
+	cptr = available_ciphers;
+	while (cptr->name) {
+	    /* find the cipher requested & make sure it's one we're happy
+	       with by policy */
+	    if (!strcmp(cipher, cptr->name) && 
+		text->requiressf <= cptr->ssf && text->limitssf >= cptr->ssf) {
+		/* found it! */
+		break;
+	    }
+	    cptr++;
+	}
 
-#ifdef WITH_RC4
-      } else if (strcmp(cipher,"rc4")==0) {
-	text->cipher_enc=(cipher_function_t *) &enc_rc4;
-	text->cipher_dec=(cipher_function_t *) &dec_rc4;
-	text->cipher_init=&init_rc4;
-	oparams->mech_ssf = 128;
- 	n = 16;
-      } else if (strcmp(cipher,"rc4-40")==0) {
- 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
- 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
- 	text->cipher_init=&init_rc4;
- 	oparams->mech_ssf = 40;
- 	n = 5;
-      } else if (strcmp(cipher,"rc4-56")==0) {
- 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
- 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
- 	text->cipher_init=&init_rc4;
- 	oparams->mech_ssf = 56;
- 	n = 7;
-#endif /* WITH_RC4 */
+	if (cptr->name) {
+	    text->cipher_enc = cptr->cipher_enc;
+	    text->cipher_dec = cptr->cipher_dec;
+	    text->cipher_init = cptr->cipher_init;
+	    oparams->mech_ssf = cptr->ssf;
+	    n = cptr->n;
+	} else {
+	    /* erg? client requested something we didn't advertise! */
+	    sparams->utils->log(sparams->utils->conn, SASL_LOG_WARNING,
+			    "DIGEST_MD5", SASL_FAIL, 0,
+		    "protocol violation: client requested invalid cipher");
+	    result = SASL_FAIL;
+	    goto FreeAllMem;
+	}
 
-      } else {
-	VL(("Invalid or no cipher chosen\n"));
+	oparams->encode=&privacy_encode;
+	oparams->decode=&privacy_decode;
+    } else if (!strcmp(qop, "auth-int") &&
+	       text->requiressf <= 1 && text->limitssf >= 1) {
+	VL(("Client requested integrity layer\n"));
+	oparams->encode = &integrity_encode;
+	oparams->decode = &integrity_decode;
+	oparams->mech_ssf = 1;
+    } else if (!strcmp(qop, "auth") && text->requiressf == 0) {
+	VL(("Client requested no layer\n"));
+	oparams->encode = NULL;
+	oparams->decode = NULL;
+	oparams->mech_ssf = 0;
+    } else {
+	sparams->utils->log(sparams->utils->conn, SASL_LOG_WARNING,
+			    "DIGEST_MD5", SASL_FAIL, 0,
+                          "protocol violation: client requested invalid qop");
 	result = SASL_FAIL;
 	goto FreeAllMem;
-      }     
-      oparams->encode=&privacy_encode;
-      oparams->decode=&privacy_decode;
-      
-    } else if (strcmp(qop, "auth-int") == 0) {
-      VL(("Client requested integrity layer\n"));
-      oparams->encode = &integrity_encode;
-      oparams->decode = &integrity_decode;
-      oparams->mech_ssf = 1;
-    } else if (strcmp(qop, "auth") == 0) {
-      VL(("Client requested no layer\n"));
-      oparams->encode = NULL;
-      oparams->decode = NULL;
-      oparams->mech_ssf = 0;
-    } else {
-      VL(("Client requested unknown layer\n"));
-      result = SASL_FAIL;
-      goto FreeAllMem;
     }
 
     /*
@@ -2567,13 +2618,16 @@ static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
   result = getsecret(getsecret_context, mech_name, "", "", &sec);
 
   /* check version */
-  if (sec!=NULL)
+  if (sec != NULL)
   {
-      if (sec->len >= 4)
-      {
+      if (sec->len >= 4) {
 	  memcpy(&tmpversion, sec->data, 4); 
 	  tmpversion = ntohl(tmpversion);
       }
+      free(sec);
+  }
+  if (result == SASL_NOUSER) {
+      return SASL_NOUSER;
   }
 
   if (tmpversion != DIGEST_MD5_VERSION)
@@ -2589,16 +2643,8 @@ static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
       return SASL_FAIL;
   }
   
-
-  if (result == SASL_NOUSER) {
-    return SASL_NOUSER;
-  }
-
   mydb_initialized = 1;
 
-  if (sec!=NULL)
-    utils->free(sec);
-
   return result;
 }
 
@@ -2746,14 +2792,11 @@ const sasl_server_plug_t plugins[] =
     "DIGEST-MD5",
 #ifdef WITH_RC4
     128,				/* max ssf */
-#else
-#ifdef WITH_DES
+#elif WITH_DES
     112,
 #else 
     0,
-#endif /* WITH_DES */
-#endif /* WITH_RC4 */
-
+#endif
     SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
     NULL,
     &server_start,
@@ -2783,9 +2826,8 @@ int sasl_server_plug_init(sasl_utils_t * utils __attribute__((unused)),
   *plugcount = 1;
   *out_version = DIGEST_MD5_VERSION;
 
-  if ( mechanism_db_filled("DIGEST-MD5",utils) != SASL_OK)
-  {
-    return SASL_NOUSER;
+  if ( mechanism_db_filled("DIGEST-MD5",utils) != SASL_OK) {
+      return SASL_NOUSER;
   }
 
   return SASL_OK;
@@ -3236,8 +3278,8 @@ c_continue_step(void *conn_context,
   }
 
   if (text->state == 2) {
-    int need = 0;
-    int musthave = 0;
+    sasl_ssf_t limit, musthave = 0;
+    sasl_ssf_t external;
     unsigned char  *digesturi = NULL;
     unsigned char  *nonce = NULL;
     unsigned char  *ncvalue = (unsigned char *) "00000001";
@@ -3245,7 +3287,7 @@ c_continue_step(void *conn_context,
     char           *qop = NULL;
     char           *qop_list = NULL;
     int             protection = 0;
-    char           *cipher = NULL;
+    char           *usecipher = NULL;
     int             ciphers=0;
     unsigned int    n = 0;
     char           *response = NULL;
@@ -3257,8 +3299,6 @@ c_continue_step(void *conn_context,
     char           *charset = NULL;
     int             result = SASL_FAIL;
     char           *client_response = NULL;
-    sasl_security_properties_t secprops;
-    int             external;
     int             user_result = SASL_OK;
     int             auth_result = SASL_OK;
     int             pass_result = SASL_OK;
@@ -3273,6 +3313,10 @@ c_continue_step(void *conn_context,
 
     in_start = in;
 
+    if (params->props.min_ssf > params->props.max_ssf) {
+	return SASL_BADPARAM;
+    }
+
     /* parse what we got */
     while (in[0] != '\0') {
 	char *name, *value;
@@ -3331,22 +3375,15 @@ c_continue_step(void *conn_context,
 		}
 
 		if (strcmp(value, "des") == 0) {
-		    VL(("Server supports DES\n"));
 		    ciphers |= CIPHER_DES;
 		} else if (strcmp(value, "3des") == 0) {
-		    VL(("Server supports 3DES\n"));
 		    ciphers |= CIPHER_3DES;
-#ifdef WITH_RC4
 		} else if (strcmp(value, "rc4") == 0) {
-		    VL(("Server supports rc4\n"));
 		    ciphers |= CIPHER_RC4;
 		} else if (strcmp(value, "rc4-40") == 0) {
-		    VL(("Server supports rc4-40\n"));
 		    ciphers |= CIPHER_RC440;
 		} else if (strcmp(value, "rc4-56") == 0) {
-		    VL(("Server supports rc4-56\n"));
 		    ciphers |= CIPHER_RC456;
-#endif /* WITH_RC4 */
 		} else {
 		    VL(("Server supports unknown cipher: %s\n", value));
 		}
@@ -3500,133 +3537,91 @@ c_continue_step(void *conn_context,
       * response | maxbuf | charset | auth-param )
      */
 
-
     /* get requested ssf */
-    secprops = params->props;
     external = params->external_ssf;
-    /*    VL(("external ssf=%d\n", external));*/
-    if ((int)params->props.min_ssf > 56 + external) {
-	return SASL_TOOWEAK;
-    } else if (params->props.min_ssf > params->props.max_ssf) {
-	return SASL_BADPARAM;
+
+    /* what do we _need_?  how much is too much? */
+    if (params->props.max_ssf > external) {
+	limit = params->props.max_ssf - external;
+    } else {
+	limit = 0;
+    }
+    if (params->props.min_ssf > external) {
+	musthave = params->props.min_ssf - external;
+    } else {
+	musthave = 0;
     }
 
-    params->utils->free(qop);
+    /* we now go searching for an option that gives us at least "musthave"
+       and at most "limit" bits of ssf. */
+    if ((limit > 1) && (protection & DIGEST_PRIVACY)) {
+	struct digest_cipher *cipher, *bestcipher;
 
-    /* need bits of layer */
-    need = params->props.max_ssf - external;
-    musthave = params->props.min_ssf - external;
-
-    if ((need > 1) &&
-	((protection & DIGEST_PRIVACY) == DIGEST_PRIVACY)) {
 	/* let's find an encryption scheme that we like */
+	cipher = available_ciphers;
+	bestcipher = NULL;
+	while (cipher->name) {
+	    /* examine each cipher we support, see if it meets our security
+	       requirements, and see if the server supports it.
+	       choose the best one of these */
+	    if ((limit >= cipher->ssf) && (musthave <= cipher->ssf) &&
+		(ciphers & cipher->flag) &&
+		(!bestcipher || (cipher->ssf > bestcipher->ssf))) {
+		bestcipher = cipher;
+	    }
+	    cipher++;
+	}
 
-      oparams->encode = &privacy_encode; 
-      oparams->decode = &privacy_decode;
-      qop = "auth-conf";
-      VL(("Using encryption layer\n"));
+	if (bestcipher) {
+	    /* we found a cipher we like */
+	    oparams->encode = &privacy_encode; 
+	    oparams->decode = &privacy_decode;
+	    oparams->mech_ssf = bestcipher->ssf;
 
-      /* Client request encryption, server supports it */
-      /* encryption */
-#ifdef WITH_RC4
-      if ((need>=128)  && (musthave <=128) && 
-	  ((ciphers & CIPHER_RC4) == CIPHER_RC4)) { /* rc4 */
-	VL(("Trying to use rc4"));
-	cipher = "rc4";
-	text->cipher_enc=(cipher_function_t *) &enc_rc4; /* uses same function both ways */
-	text->cipher_dec=(cipher_function_t *) &dec_rc4;
-	text->cipher_init=&init_rc4;
-	oparams->mech_ssf = 128;
-	n=16;
-#else
-      if (0) {
-#endif /* WITH_RC4 */
-
-#ifdef WITH_DES
-      } else if ((need>=112) && (musthave <=112) &&
-		 ((ciphers & CIPHER_3DES) == CIPHER_3DES)) {
-	VL(("Trying to use 3des"));
-	cipher = "3des";
-	text->cipher_enc=(cipher_function_t *) &enc_3des;
-	text->cipher_dec=(cipher_function_t *) &dec_3des;
-	text->cipher_init=(cipher_init_t *) &init_3des;
-	oparams->mech_ssf = 112; 
-	n=16; /* number of bits to use for privacy key */
-#endif /* WITH_DES */
-
-#ifdef WITH_RC4
-      } else if ((need>=56) && (musthave <=56) &&
-		 ((ciphers & CIPHER_RC456) == CIPHER_RC456)) { /* rc4-56 */
- 	VL(("Trying to use rc4-56"));
- 	cipher = "rc4-56";
- 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
- 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
- 	text->cipher_init=&init_rc4;
- 	oparams->mech_ssf = 56;
- 	n = 7;
-#endif /* WITH_RC4 */
-
-#ifdef WITH_DES
-      } else if ((need>=55) && (musthave <=55) &&
-		 ((ciphers & CIPHER_DES) == CIPHER_DES)) { /* des */
-	VL(("Trying to use des"));
-	cipher = "des";
-	text->cipher_enc=(cipher_function_t *) &enc_des;
-	text->cipher_dec=(cipher_function_t *) &dec_des;
-	text->cipher_init=(cipher_init_t *) &init_des;
-	oparams->mech_ssf = 55; 
-	n=16;
-#endif /* WITH_DES */
-
-#ifdef WITH_RC4
-      } else if ((need>=40) && (musthave <=40) &&
-		 ((ciphers & CIPHER_RC440) == CIPHER_RC440)) { /* rc4-40 */
- 	VL(("Trying to use rc4-40"));
- 	cipher = "rc4-40";
- 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
- 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
- 	text->cipher_init=&init_rc4;
- 	oparams->mech_ssf = 40;
- 	n = 5;
- 
-#endif /* WITH_RC4 */
-
-      } else {
-	/* should try integrity or plain */
-	VL(("No good privacy layers\n"));
-	qop=NULL;
-      }
+	    qop = "auth-conf";
+	    n = bestcipher->n;
+	    usecipher = bestcipher->name;
+	    text->cipher_enc = bestcipher->cipher_enc;
+	    text->cipher_dec = bestcipher->cipher_dec;
+	    text->cipher_init = bestcipher->cipher_init;
+	} else {
+	    /* we didn't find any ciphers we like */
+	    VL(("No good privacy layers\n"));
+	    qop = NULL;
+	}
     }
 
-    if (qop==NULL)
-    {
-      if ((need >= 1) && (musthave <=1) &&
-	  ((protection & DIGEST_INTEGRITY) == DIGEST_INTEGRITY)) {
-	/* integrity */
-	oparams->encode = &integrity_encode;
-	oparams->decode = &integrity_decode;
-	oparams->mech_ssf = 1;
-	qop = "auth-int";
-	VL(("Using integrity layer\n"));
-      } else if (musthave <=0) {
-	/* no layer */
-	oparams->encode = NULL;
-	oparams->decode = NULL;
-	oparams->mech_ssf = 0;
-	qop = "auth";
-	VL(("Using no layer\n"));
+    if (qop==NULL) {
+	/* we failed to find an encryption layer we liked;
+	   can we use integrity or nothing? */
 
-	/* See if server supports not having a layer */
-	if ((protection & DIGEST_NOLAYER) != DIGEST_NOLAYER) {
-	  VL(("Server doesn't support \"no layer\"\n"));
-	  result = SASL_FAIL;
-	  goto FreeAllocatedMem;
+	if ((limit >= 1) && (musthave <= 1) 
+	    && (protection & DIGEST_INTEGRITY)) {
+	    /* integrity */
+	    oparams->encode = &integrity_encode;
+	    oparams->decode = &integrity_decode;
+	    oparams->mech_ssf = 1;
+	    qop = "auth-int";
+	    VL(("Using integrity layer\n"));
+	} else if (musthave <= 0) {
+	    /* no layer */
+	    oparams->encode = NULL;
+	    oparams->decode = NULL;
+	    oparams->mech_ssf = 0;
+	    qop = "auth";
+	    VL(("Using no layer\n"));
+	    
+	    /* See if server supports not having a layer */
+	    if ((protection & DIGEST_NOLAYER) != DIGEST_NOLAYER) {
+		VL(("Server doesn't support \"no layer\"\n"));
+		result = SASL_FAIL;
+		goto FreeAllocatedMem;
+	    }
+	} else {
+	    VL(("Can't find an acceptable layer\n"));
+	    result = SASL_TOOWEAK;
+	    goto FreeAllocatedMem;
 	}
-      } else {
-	  VL(("Can't find an acceptable layer\n"));
-	  result = SASL_FAIL;
-	  goto FreeAllocatedMem;
-      }
     }
 
     /* get nonce XXX have to clean up after self if fail */
@@ -3709,9 +3704,9 @@ c_continue_step(void *conn_context,
       result = SASL_FAIL;
       goto FreeAllocatedMem;
     }
-    if (cipher!=NULL)
+    if (usecipher!=NULL)
       if (add_to_challenge(params->utils, &client_response, "cipher", 
-			   (unsigned char *) cipher, TRUE) != SASL_OK) {
+			   (unsigned char *) usecipher, TRUE) != SASL_OK) {
 	result = SASL_FAIL;
 	goto FreeAllocatedMem;
       }
@@ -3894,13 +3889,11 @@ const sasl_client_plug_t client_plugins[] =
     "DIGEST-MD5",
 #ifdef WITH_RC4
     128,				/* max ssf */
-#else
-#ifdef WITH_DES
+#elif WITH_DES
     112,
 #else
     0,
-#endif /* WITH_DES */
-#endif /* WITH_RC4 */
+#endif
     SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
     client_required_prompts,
     NULL,
