@@ -81,7 +81,8 @@ typedef struct context {
     gss_name_t   client_name;
     gss_name_t   server_name;
     gss_cred_id_t server_creds;
-    sasl_ssf_t ssf; /* security layer type */
+    sasl_ssf_t limitssf, requiressf; /* application defined bounds, for the
+					server */
 
     sasl_malloc_t *malloc;	/* encode and decode need these */
     sasl_realloc_t *realloc;       
@@ -214,7 +215,7 @@ sasl_gss_integrity_encode(void *context, const char *input, unsigned inputlen,
   return sasl_gss_encode(context,input,inputlen,output,outputlen,0);
 }
 
-#define MIN(a,b) (((a) < (b)) ? (a) : (b))
+#define myMIN(a,b) (((a) < (b)) ? (a) : (b))
 
 static int 
 sasl_gss_decode(void *context, const char *input, unsigned inputlen,
@@ -232,7 +233,7 @@ sasl_gss_decode(void *context, const char *input, unsigned inputlen,
     /* first we need to extract a packet */
     if (text->needsize > 0) {
 	/* how long is it? */
-	int tocopy = MIN(text->needsize, inputlen);
+	int tocopy = myMIN(text->needsize, inputlen);
 
 	memcpy(text->sizebuf + 4 - text->needsize, input, tocopy);
 	text->needsize -= tocopy;
@@ -558,7 +559,30 @@ sasl_gss_server_step (void *conn_context,
 
 	oparams->authid = (char *)name_token.value;
 
-	sasldata[0] = 1 | 2 | 4;
+	/* we have to decide what sort of encryption/integrity/etc.,
+	   we support */
+	if (params->props.max_ssf < params->external_ssf) {
+	    text->limitssf = 0;
+	} else {
+	    text->limitssf = params->props.max_ssf - params->external_ssf;
+	}
+	if (params->props.min_ssf < params->external_ssf) {
+	    text->requiressf = 0;
+	} else {
+	    text->requiressf = params->props.min_ssf - params->external_ssf;
+	}
+
+	sasldata[0] = 0;
+	if (text->requiressf == 0) {
+	    sasldata[0] |= 1; /* authentication */
+	}
+	if (text->requiressf <= 1 && text->limitssf >= 1) {
+	    sasldata[0] |= 2;
+	}
+	if (text->requiressf <= 56 && text->limitssf >= 56) {
+	    sasldata[0] |= 4;
+	}
+
 	sasldata[1] = 0x0F; /* XXX use something non-artificial */
 	sasldata[2] = 0xFF;
 	sasldata[3] = 0xFF;
@@ -598,6 +622,8 @@ sasl_gss_server_step (void *conn_context,
       }
     case SASL_GSSAPI_STATE_SSFREQ:
       {
+	int layerchoice;
+
 	real_input_token.value = (void *)clientin;
 	real_input_token.length = clientinlen;
 	
@@ -617,28 +643,29 @@ sasl_gss_server_step (void *conn_context,
 	    return SASL_FAIL;
 	}
 	
-	text->ssf = (int)(((char *)(output_token->value))[0]);
-	if (text->ssf==1)  /* no encryption */
-	  {
-	    oparams->encode=NULL;
-	    oparams->decode=NULL;
-	    oparams->mech_ssf=0;
-	    text->ssf=1;
-	  } else if (text->ssf==2) { /* integrity */
+	layerchoice = (int)(((char *)(output_token->value))[0]);
+	if (layerchoice == 1 && text->requiressf == 0) { /* no encryption */
+	    oparams->encode = NULL;
+	    oparams->decode = NULL;
+	    oparams->mech_ssf = 0;
+	} else if (layerchoice == 2 && text->requiressf <= 1 &&
+		   text->limitssf >= 1) { /* integrity */
 	    oparams->encode=&sasl_gss_integrity_encode;
 	    oparams->decode=&sasl_gss_decode;
 	    oparams->mech_ssf=1;
-	    text->ssf=2;
-	  } else if (text->ssf==4) { /* privacy */
-	    oparams->encode=&sasl_gss_privacy_encode;
-	    oparams->decode=&sasl_gss_decode;
-	    oparams->mech_ssf=56;
-	    text->ssf=4;
-	  } else {
+	} else if (layerchoice == 4 && text->requiressf <= 56 &&
+		   text->limitssf >= 56) { /* privacy */
+	    oparams->encode = &sasl_gss_privacy_encode;
+	    oparams->decode = &sasl_gss_decode;
+	    oparams->mech_ssf = 56;
+	} else {
 	    /* not a supported encryption layer */
+	    params->utils->log(params->utils->conn, SASL_LOG_WARNING,
+			       "GSSAPI", SASL_FAIL, 0, 
+		    "protocol violation: client requested invalid layer");
 	    sasl_gss_free_context_contents(text);
 	    return SASL_FAIL;
-	  }
+	}
 	DEBUG ((stderr,"Got %d bytes from client\n",output_token->length));
 	if (output_token->length > 4) {
 	    char *user = (char *)params->utils->malloc(
@@ -1083,7 +1110,8 @@ sasl_gss_client_step (void *conn_context,
       {
 	sasl_security_properties_t secprops = params->props;
 	unsigned int alen, external = params->external_ssf;
-	int need = 0;
+	sasl_ssf_t need, allowed;
+	char serverhas, mychoice;
 
 	if (serverinlen)
 	  {
@@ -1109,46 +1137,44 @@ sasl_gss_client_step (void *conn_context,
 	  }
 	
 	/* taken from kerberos.c */
-	if (secprops.min_ssf > 56 + external) {
+	if (secprops.min_ssf > (56 + external)) {
 	    return SASL_TOOWEAK;
 	} else if (secprops.min_ssf > secprops.max_ssf) {
 	    return SASL_BADPARAM;
 	}
 
 	/* need bits of layer */
-	need = secprops.max_ssf - external;
+	allowed = secprops.max_ssf - external;
+	need = secprops.min_ssf - external;
+	serverhas = ((char *)output_token->value)[0];
 
-	/* if client didn't set use strongest layer */
-	if (need >= 56) {
+	/* if client didn't set use strongest layer available */
+	if (allowed >= 56 && need <= 56 && (serverhas & 4)) {
 	    /* encryption */
 	    oparams->encode = &sasl_gss_privacy_encode;
 	    oparams->decode = &sasl_gss_decode;
 	    oparams->mech_ssf = 56;
-	    text->ssf = 4;
+	    mychoice = 4;
 	    DEBUG ((stderr,"Using encryption layer\n"));
-	} else if (need >= 1) {
+	} else if (allowed >= 1 && need <= 1 && (serverhas & 2)) {
 	    /* integrity */
 	    oparams->encode = &sasl_gss_integrity_encode;
 	    oparams->decode = &sasl_gss_decode;
 	    oparams->mech_ssf = 1;
-	    text->ssf = 2;
+	    mychoice = 2;
 	    DEBUG ((stderr,"Using integrity layer\n"));
-	} else {
+	} else if (need <= 0 && (serverhas & 1)) {
 	    /* no layer */
 	    oparams->encode = NULL;
 	    oparams->decode = NULL;
 	    oparams->mech_ssf = 0;
-	    text->ssf = 1;
+	    mychoice = 1;
 	    DEBUG ((stderr,"Using no layer\n"));
-	}
-
-	/* server told me what layers support. 
-	   make sure trying one it supports */
-	if ( (((char *)output_token->value)[0] & text->ssf) == 0 )
-	  {
+	} else {
+	    /* there's no appropriate layering for us! */
 	    sasl_gss_free_context_contents(text);
-	    return SASL_WRONGMECH;
-	  }
+	    return SASL_TOOWEAK;
+	}
 	
 	params->utils->free(output_token->value);
 	output_token->value = NULL;
@@ -1165,7 +1191,7 @@ sasl_gss_client_step (void *conn_context,
 	memcpy((char *)input_token->value+4,oparams->user,alen);
 	
 	
-	((unsigned char *)input_token->value)[0] = text->ssf;
+	((unsigned char *)input_token->value)[0] = mychoice;
 	oparams->maxoutbuf = 1024; /* XXX do something real here */
 	((unsigned char *)input_token->value)[1] = 0x0F;
 	((unsigned char *)input_token->value)[2] = 0xFF;
